@@ -322,12 +322,12 @@ class ValidIDREF(base.BaseRule):
 
   def elements(self):
     schema_tree = etree.parse(self.schema_file)
-    eligible_elements = []
+    eligible_elements = set()
     for _, element in etree.iterwalk(schema_tree):
       tag = self.strip_schema_ns(element)
       if (tag and tag == "element" and
           element.get("type") in ("xs:IDREF", "xs:IDREFS")):
-        eligible_elements.append(element.get("name"))
+        eligible_elements.add(element.get("name"))
     return eligible_elements
 
   def check(self, element):
@@ -1433,6 +1433,32 @@ class CandidatesMissingPartyData(base.BaseRule):
           (element.sourceline, element.get("objectId")))
 
 
+class CandidatesReferencePerson(base.ValidReferenceRule):
+  """Each candidate must reference a valid person via the PersonId field."""
+
+  def __init__(self, election_tree, schema_file):
+    super(CandidatesReferencePerson,
+          self).__init__(election_tree, schema_file, "Person")
+
+  def _gather_reference_values(self):
+    reference_values = set()
+    candidates = self.get_elements_by_class(self.election_tree, "Candidate")
+
+    for candidate in candidates:
+      person_id = candidate.find("PersonId")
+      if person_id is not None and person_id.text is not None:
+        reference_values.add(person_id.text)
+
+    return reference_values
+
+  def _gather_defined_values(self):
+    all_people = set()
+    person_collection = self.election_tree.getroot().find("PersonCollection")
+    if person_collection is not None:
+      all_people = {person.attrib["objectId"] for person in person_collection}
+    return all_people
+
+
 class OfficeMissingOfficeHolderPersonData(base.ValidReferenceRule):
   """Each Office must have Persons occupying it.
 
@@ -2212,6 +2238,32 @@ class ImproperCandidateContest(base.TreeRule):
                                     error_log)
 
 
+class RequiredFields(base.BaseRule):
+  """Check for required fields for given entity types and field names.
+
+  To add a field, include the entity and field in the _element_field_mapping
+  and add the entity to the elements list.
+  """
+
+  _element_field_mapping = {
+      "Person": "FullName//Text",
+      "Candidate": "PersonId",
+  }
+
+  def elements(self):
+    return ["Person", "Candidate"]
+
+  def check(self, element):
+    required_field_tag = self._element_field_mapping[element.tag]
+    required_field = element.find(required_field_tag)
+    if (required_field is None or required_field.text is None
+        or not required_field.text.strip()):
+      raise loggers.ElectionError(
+          "Line {}. Element {} (objectId: {}) is missing required "
+          "field {}.".format(element.sourceline, element.tag,
+                             element.get("objectId"), required_field_tag))
+
+
 class RuleSet(enum.Enum):
   """Names for sets of rules used to validate a particular feed type."""
   ELECTION = 1
@@ -2252,6 +2304,7 @@ COMMON_RULES = (
     PersonHasUniqueFullName,
     PersonsMissingPartyData,
     GpUnitsHaveInternationalizedName,
+    RequiredFields,
 )
 
 ELECTION_RULES = COMMON_RULES + (
@@ -2280,6 +2333,7 @@ ELECTION_RULES = COMMON_RULES + (
     FullTextOrBallotText,
     BallotTitle,
     ImproperCandidateContest,
+    CandidatesReferencePerson,
 )
 
 OFFICEHOLDER_RULES = COMMON_RULES + (
